//   This sketch loads and runs .  The app finds it but errors out upon attempted connection.

// 30/05/2024 13:15:06  Chris Ward
//BME_SensorObject.ino

//https://wiki.seeedstudio.com/XIAO_ESP32C3_Getting_Started

#include <ArduinoBLE.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>

#define BLE_NAME "Original_BME_SensorObject"

//https://www.professorcad.co.uk/appinventortips#BleUuids
#define BLE_SERVICE_UUID "0357A1D0-58C9-48E5-8C81-3DA5D49C5C21"
#define BLE_CHARACTERISTIC_UUID "0357A1D1-58C9-48E5-8C81-3DA5D49C5C21"

// Create BME280 sensor object
Adafruit_BME280 bme;

// Service
BLEService bmeService(BLE_SERVICE_UUID);
// Characteristic
BLEStringCharacteristic bmeCharacteristic(BLE_CHARACTERISTIC_UUID, BLEWrite | BLENotify, 32); 

// Global Values
float Temperature;
float Humidity;
float Pressure;
long lgPeriod = 2000;         // Check period (first run 2 seconds, subsequent 10 seconds)
bool bgUpdate = false;        // Subseqent check period 10 seconds                        
long lgPreviousMillis = 0;    // last time the sensor values were checked, in ms     

void setup()
{
               Serial.begin(9600);

               //while (!Serial);  //  Comment out this line to work as a stand-alone project without the IDE attached.
              
               if (!BLE.begin())
               {
                         Serial.println("Failed to start BLE!");
                         while (1);
               }
              
               if (!bme.begin(0x76))
               {
                         Serial.println("Could not find a valid BME280 sensor, check wiring!");
                         while (1);
               }
              
               BLE.setLocalName(BLE_NAME);
               BLE.setAdvertisedService(bmeService);
               BLE.addService(bmeService);
               
               bmeService.addCharacteristic(bmeCharacteristic);
              
               BLE.advertise();
              
               Serial.println("BLE peripheral is now active");
}

void updateValues()
{
      char sSensorVals[32]; // Buffer

               // To Serial Monitor
               Serial.print("Temperature: "); Serial.print(Temperature); Serial.println(" °C");
               Serial.print("Humidity: ");    Serial.print(Humidity);    Serial.println(" %");
               Serial.print("Pressure: ");    Serial.print(Pressure);    Serial.println(" hPa");
              
               sprintf(sSensorVals, "%.2f|%.2f|%.2f\0", Temperature, Humidity, Pressure); 
               bmeCharacteristic.writeValue(sSensorVals); //To App
}

void loop() 
{
               BLEDevice central = BLE.central();
               
               if (central)
               {
                         Serial.print("Connected to central: "); Serial.println(central.address());
               
                         while (central.connected())
                         {

                                   // if lgPeriod milliseconds have passed, check the sensor values:
                                   if ((millis() - lgPreviousMillis) > lgPeriod)
                                   {
                         
                                             // Read actual sensor values
                                             Temperature = bme.readTemperature();
                                                Humidity = bme.readHumidity();
                                                Pressure = (bme.readPressure() / 100.0); // Convert Pa to hPa
                                             
                                             updateValues();

                                             bgUpdate = true;
                                   }

                                   if (bgUpdate == true) {lgPeriod = 10000;} //Subsequent checks every 10 seconds
                         }
               
                         Serial.print("Disconnected from central: ");
                         Serial.println(central.address());
               }
}
